// SqlSvc
// SQL-in-a-Service example (tested with Interbase)
//
// Author:     Peter Sawatzki (ps)
//             Buchenhof 3, 58091 Hagen, Germany
// CompuServe: 100031,3002
// E-Mail:     Peter@Sawatzki.de
// web:        http://www.sawatzki.de
//
// Date:    Author:
// 04/18/97 ps     first version for Delphi 2/3
//
// Copyright (c) 1996, 1997 Peter Sawatzki. All Rights Reserved.
//
// usage instructions:
//  - compile with "dcc32 -cc SqlSvc" or use Delphi IDE, but set linker
//    options to "Generate console application"
//  - for this demo to work, create a database/alias 'SqlSvcDb'
//  - for Interbase make sure that the alias refers to a *remote* database (even
//    if you access a local database on the same system this service runs)
//    so let SqlSvcDb point to something like 'guru:/db/SqlSvcDb.gdb' and NOT
//    'd:/db/SqlSvcDb.gdb'
//  - create a table 'TIMETABLE' [CREATE TABLE TIMETABLE (TIME VARCHAR(40));]
//    in the database and a user 'TESTUSER' with password 'secret' that has access
//    to the table [GRANT ALL ON TIMETABLE TO TESTUSER]
//
Program SqlSvc;
Uses
  SysUtils, Windows, Classes,
  Db, DbTables,
  SvcObj, NTObj;
Type
  TSqlService = Class(TService)
  Private
    ATable: TTable;
    ASession: TSession;
    ADatabase: TDatabase;
  Protected
    Procedure Execute; Override;
  End;

Procedure TSqlService.Execute;
Begin
  Try
    ASession:= TSession.Create(Nil);
    Try
      ASession.SessionName:= 'Session'+IntToStr(GetCurrentThreadId);
      ADatabase:= TDatabase.Create(Nil);
      Try
        ADatabase.AliasName:= 'SqlSvcDb';
        ADatabase.DatabaseName:= 'SqlSvcDbDataBase';
        ADatabase.SessionName:= ASession.SessionName;
        ADatabase.Params.Add('USER NAME=TESTUSER');
        ADatabase.Params.Add('PASSWORD=secret');
        ADatabase.LoginPrompt:= False;
        ATable:= TTable.Create(Nil);
        Try
          ATable.DatabaseName:= 'SqlSvcDbDataBase';
          ATable.TableName:= 'TIMETABLE';
          ATable.SessionName:= ASession.SessionName;

          ADatabase.Connected:= True;
          ATable.Open;
          // stuff current time into table every ten seconds
          While WaitForOneObject([Self, StopEvent], 10000)<>WAIT_OBJECT_0+1 Do Begin
            ATable.Append;
            ATable.FieldByName('Time').AsString:= DateTimeToStr(Now);
            ATable.Post
          End;
          ATable.Close;
        Finally
          ATable.Free
        End
      Finally
        ADatabase.Free
      End
    Finally
      ASession.Free
    End
  Except
    On E: Exception Do
      AddToEventLog(elException, Format('%s raised: %s', [E.ClassName, E.Message]));
  End
End;

Begin
  TSqlService.Create('ASqlSvc', 'A SQL Service');
  CmdLine;
  WaitForServiceCompletion
End.
